/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator;

import java.util.HashSet;
import java.util.Objects;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;

import org.keycloak.admin.client.Keycloak;
import org.reflections.Reflections;
import org.springframework.aot.hint.MemberCategory;
import org.springframework.aot.hint.RuntimeHints;
import org.springframework.aot.hint.RuntimeHintsRegistrar;

import io.fabric8.kubernetes.api.model.KubernetesResource;
import io.fabric8.kubernetes.api.model.NamedCluster;
import lombok.extern.slf4j.Slf4j;

//@Configuration
//@ImportRuntimeHints(SpringNativeConfiguration.KuberenetesCLientImplHints.class)
@Slf4j
public class SpringNativeConfiguration {

	static class KuberenetesCLientImplHints implements RuntimeHintsRegistrar {

		private final Class<?> clazz = NamedCluster.class;
		private final Reflections reflections = new Reflections(clazz.getPackageName(), clazz);

		@Override
		public void registerHints(RuntimeHints hints, ClassLoader classLoader) {
			// https://github.com/kubernetes-native-java/fabric8-spring-native/
			var subtypesOfKubernetesResource = reflections.getSubTypesOf(KubernetesResource.class);

			var combined = new HashSet<Class<?>>();
			combined.addAll(subtypesOfKubernetesResource);
			combined.add(io.fabric8.kubernetes.internal.KubernetesDeserializer.class);
			combined.add(io.fabric8.kubernetes.client.impl.KubernetesClientImpl.class);
			combined.add(io.javaoperatorsdk.operator.processing.retry.GenericRetry.class);
			combined.add(io.javaoperatorsdk.operator.processing.event.rate.LinearRateLimiter.class);
//			Class.forName("io.fabric8.kubernetes.client.okhttp.OkHttpClientBuilderImpl");
//			reflections.getSubTypesOf(io.fabric8.kubernetes.client.okhttp.OkHttpClientBuilderImpl.class);

//			combined.add(OzgKeycloakUser.class);
//			combined.add(OzgKeycloakUserSpec.class);
//			combined.add(OzgKeycloakUserSpec.KeycloakUserSpecUser.class);
//			combined.add(OzgKeycloakUserSpec.KeycloakUserSpecClientRole.class);
//			combined.add(OzgKeycloakUserSpec.KeycloakUserSpecUserGroup.class);
//			combined.add(OzgKeycloakUserStatus.class);

			combined.add(Keycloak.class);
			combined.add(org.keycloak.admin.client.spi.ResteasyClientClassicProvider.class);
			combined.add(org.jboss.resteasy.client.jaxrs.ResteasyClientBuilder.class);
			combined.addAll(subtypesOfKubernetesResource);

			combined.add(org.jboss.resteasy.resteasy_jaxrs.i18n.Messages.class);
			combined.addAll(reflections.getSubTypesOf(org.jboss.resteasy.resteasy_jaxrs.i18n.Messages.class));
			combined.add(org.jboss.resteasy.resteasy_jaxrs.i18n.Messages_$bundle.class);
			combined.add(org.jboss.resteasy.resteasy_jaxrs.i18n.LogMessages.class);
			combined.add(org.jboss.resteasy.resteasy_jaxrs.i18n.LogMessages_$logger.class);
			combined.addAll(reflections.getSubTypesOf(org.jboss.resteasy.resteasy_jaxrs.i18n.LogMessages.class));

			combined.add(org.jboss.resteasy.client.jaxrs.i18n.LogMessages.class);
			combined.addAll(reflections.getSubTypesOf(org.jboss.resteasy.client.jaxrs.i18n.LogMessages.class));

			combined.add(org.jboss.resteasy.plugins.providers.jaxb.i18n.LogMessages.class);
			combined.add(org.jboss.resteasy.plugins.providers.jaxb.i18n.LogMessages_$logger.class);

			combined.add(org.jboss.resteasy.plugins.providers.RegisterBuiltin.class);

			combined.add(org.jboss.resteasy.client.jaxrs.internal.ResteasyClientBuilderImpl.class);

			combined.add(org.keycloak.admin.client.Keycloak.class);

			combined.add(org.keycloak.admin.client.JacksonProvider.class);

			try {
				combined.add(Class.forName("io.fabric8.kubernetes.client.okhttp.OkHttpClientBuilderImpl"));
			} catch (ClassNotFoundException e) {
				throw new RuntimeException(e);
			}

			combined.add(ThreadPoolExecutor.class);
			combined.addAll(reflections.getSubTypesOf(ExecutorService.class));

			combined
					.stream()
					.filter(Objects::nonNull)
					.forEach(c -> {
						if (log.isDebugEnabled()) {
							log.debug("trying to register " + c.getName() + " for reflection");
						}
						hints.reflection().registerType(c, MemberCategory.values());

//						registry.reflection().forType(c).withAccess(TypeAccess.values()).build();
					});
//			hints.reflection().registerType(clazz, null)
//					.registerType(KubernetesClientImpl.class, MemberCategory.INVOKE_PUBLIC_CONSTRUCTORS);

		}
	}
}
