package de.ozgcloud.operator.keycloak.client;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;

import org.keycloak.representations.idm.ProtocolMapperRepresentation;
import org.springframework.stereotype.Component;

import lombok.extern.java.Log;

@Log
@Component
class ProtocolMapperRepresentationHelper {

	static final String OPENID_CONNECT_PROTOCOL = "openid-connect";
	static final String ATTRIBUTE_PROTOCOL_MAPPER = "oidc-usermodel-attribute-mapper";

	static final String CONFIG_ACCESS_TOKEN_CLAIM_KEY = "access.token.claim";
	static final String CONFIG_AGGREGATE_ATTRS_KEY = "aggregate.attrs";
	static final String CONFIG_CLAIM_NAME_KEY = "claim.name";
	static final String CONFIG_ID_TOKEN_CLAIM_KEY = "id.token.claim";
	static final String CONFIG_JSON_TYPE_LABEL_KEY = "jsonType.label";
	static final String CONFIG_MULTIVALUED_KEY = "multivalued";
	static final String CONFIG_USER_ATTRIBUTE_KEY = "user.attribute";
	static final String CONFIG_USERINFO_TOKEN_CLAIM_KEY = "userinfo.token.claim";

	static final String ORGANISATIONS_EINHEIT_ID_MAPPER_NAME = "organisationseinheitIdMapper";

	public ProtocolMapperRepresentation createOrganisationsEinheitIdMapper() {
		log.log(Level.FINE, "Build organisationsEinheitId mapper...");
		var mapper = new ProtocolMapperRepresentation();
		mapper.setName(ORGANISATIONS_EINHEIT_ID_MAPPER_NAME);
		mapper.setProtocol(OPENID_CONNECT_PROTOCOL);
		mapper.setProtocolMapper(ATTRIBUTE_PROTOCOL_MAPPER);
		mapper.setConfig(buildOrganisationsEinheitMapperConfig());

		return mapper;
	}

	Map<String, String> buildOrganisationsEinheitMapperConfig() {
		var config = new HashMap<String, String>();
		config.put(CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
		config.put(CONFIG_AGGREGATE_ATTRS_KEY, "true");
		config.put(CONFIG_CLAIM_NAME_KEY, "organisationseinheitId");
		config.put(CONFIG_ID_TOKEN_CLAIM_KEY, "true");
		config.put(CONFIG_JSON_TYPE_LABEL_KEY, "int");
		config.put(CONFIG_MULTIVALUED_KEY, "true");
		config.put(CONFIG_USER_ATTRIBUTE_KEY, "organisationseinheitId");
		config.put(CONFIG_USERINFO_TOKEN_CLAIM_KEY, "true");
		return config;
	}
}