/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.group;

import java.util.Objects;
import java.util.Optional;
import java.util.logging.Level;

import javax.ws.rs.core.Response;

import org.keycloak.admin.client.Keycloak;
import org.keycloak.representations.idm.GroupRepresentation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.keycloak.KeycloakResultParser;
import lombok.extern.java.Log;

@Component
@Log
class KeycloakGroupRemoteService {

	@Autowired
	private Keycloak keycloak;

	Optional<GroupRepresentation> getGroupByName(String groupName, String realm) {
		return keycloak.realm(realm).groups().groups()
				.stream().filter(group -> Objects.equals(groupName, group.getName()))
				.findFirst();
	}

	void createGroup(GroupRepresentation group, String realm) {
		log.log(Level.FINE, "Creating group {0} in realm {1}", new Object[] { group.getName(), realm });
		Response response = keycloak.realm(realm).groups().add(group);
		KeycloakResultParser.parseCreatedResponse(response);
	}
}
