package de.ozgcloud.operator.keycloak.client;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.representations.idm.ProtocolMapperRepresentation;
import org.mockito.Spy;

class ProtocolMapperRepresentationHelperTest {

	@Spy
	private final ProtocolMapperRepresentationHelper helper = new ProtocolMapperRepresentationHelper();

	@DisplayName("Create organisationsEinheitId mapper")
	@Nested
	class TestCreateOrganisationsEinheitIdMapper {

		@Test
		void shouldSetName() {
			var mapper = create();

			assertThat(mapper.getName()).isEqualTo(ProtocolMapperRepresentationHelper.ORGANISATIONS_EINHEIT_ID_MAPPER_NAME);
		}

		@Test
		void shouldSetProtocol() {
			var mapper = create();

			assertThat(mapper.getProtocol()).isEqualTo(ProtocolMapperRepresentationHelper.OPENID_CONNECT_PROTOCOL);
		}

		@Test
		void shouldSetProtocolMapper() {
			var mapper = create();

			assertThat(mapper.getProtocolMapper()).isEqualTo(ProtocolMapperRepresentationHelper.ATTRIBUTE_PROTOCOL_MAPPER);
		}

		@Test
		void shouldBuildConfig() {
			create();

			verify(helper).buildOrganisationsEinheitMapperConfig();
		}

		@DisplayName("build config")
		@Nested
		class TestBuildOrganisationsEinheitIdConfig {

			@Test
			void shouldSetAccessTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ACCESS_TOKEN_CLAIM_KEY, "true");
			}

			@Test
			void shouldSetAggregateAttrs() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_AGGREGATE_ATTRS_KEY, "true");
			}

			@Test
			void shouldSetClaimName() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_CLAIM_NAME_KEY, "organisationseinheitId");
			}

			@Test
			void shouldSetIdTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_ID_TOKEN_CLAIM_KEY, "true");
			}

			@Test
			void shouldSetJsonTypeLabel() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_JSON_TYPE_LABEL_KEY, "int");
			}

			@Test
			void shouldSetMultivalued() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_MULTIVALUED_KEY, "true");
			}

			@Test
			void shouldSetUserAttribute() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USER_ATTRIBUTE_KEY, "organisationseinheitId");
			}

			@Test
			void shouldSetUserInfoTokenClaim() {
				var config = buildConfig();

				assertThat(config).containsEntry(ProtocolMapperRepresentationHelper.CONFIG_USERINFO_TOKEN_CLAIM_KEY, "true");
			}

			private Map<String, String> buildConfig() {
				return helper.buildOrganisationsEinheitMapperConfig();
			}
		}

		private ProtocolMapperRepresentation create() {
			return helper.createOrganisationsEinheitIdMapper();
		}
	}
}
