/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.OzgCustomResourceStatus;

class KeycloakUserReconcilerTest {

	@Spy
	@InjectMocks
	private KeycloakUserReconciler conciler;

	@Mock
	private KeycloakUserService userService;

	@Mock
	private KeycloakUserPreconditionService preconditionService;

	@Nested
	class TestAddUser {

		@BeforeEach
		void init() {
			when(preconditionService.getReconcilePreconditionErrors(any())).thenReturn(Optional.empty());
		}

		@Test
		void shouldCallServiceAddUser() {
			OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();

			conciler.reconcile(user, null);

			verify(userService).createOrUpdateUser(user.getSpec(), OzgKeycloakUserTestFactory.METADATA_NAMESPACE);
		}

		@Test
		void shouldUpdateStatus() {
			OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();

			conciler.reconcile(user, null);

			assertThat(user.getStatus().getStatus()).isEqualTo(OzgCustomResourceStatus.OK);
		}

		@Nested
		class TestPreconditionError {

			private static final String ERROR_MESSAGE = "ErrorMessage";

			@Test
			void shouldSetPreconcitionErrorMessage() {
				when(preconditionService.getReconcilePreconditionErrors(any())).thenReturn(Optional.of(ERROR_MESSAGE));
				OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();

				conciler.reconcile(user, null);

				assertThat(user.getStatus().getMessage()).isEqualTo(ERROR_MESSAGE);
			}
		}
	}

	@Nested
	@Disabled("ErrorStatusHandler erstmal deaktiviert weil der nicht funktioniert hat")
	class TestErrorStatusUpdateControl {

		@Test
		void shouldSetErrorStatusOnException() {
			OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();
			doThrow(new RuntimeException()).when(userService).createOrUpdateUser(any(), any());

			conciler.reconcile(user, null);

			assertThat(user.getStatus().getStatus()).isEqualTo(OzgCustomResourceStatus.ERROR);
		}
	}

	@Nested
	class TestDeleteUser {

		@Test
		void shouldCallServiceDeleteUser() {
			OzgKeycloakUser user = OzgKeycloakUserTestFactory.create();

			conciler.cleanup(user, null);

			verify(userService).deleteUser(user.getSpec(), OzgKeycloakUserTestFactory.METADATA_NAMESPACE);
		}
	}
}
